#!/usr/bin/env __NV_PRIME_RENDER_OFFLOAD=1 __VK_LAYER_NV_optimus=NVIDIA_only __GLX_VENDOR_LIBRARY_NAME=nvidia python

import numpy as np
from mayavi import mlab
from mayavi.core.ui.api import MayaviScene, MlabSceneModel, SceneEditor
from mayavi.tools.animator import Animator
from traits.api import HasTraits, Instance, Range, on_trait_change
from traitsui.api import Group, Item, View

π = np.pi

xy = np.linspace(0., 1., 201)
xx, yy = np.meshgrid(xy, xy)
xx, yy = xx.T, yy.T

dt = 0.05
de = np.exp(1j * dt * π)


class Mode(HasTraits):
    m = Range(0, 10, 1)
    n = Range(0, 10, 1)

    scene = Instance(MlabSceneModel, ())

    view = View(
        Item('scene', editor=SceneEditor(scene_class=MayaviScene), show_label=False),
        Group(Group(
            Item("m"),
            Item("n"),
        ), Item("animator", show_label=False, style='custom'), orientation='horizontal'),
        resizable=True,
    )

    animator = Instance(Animator)

    def _animator_default(self):
        return self.animate()

    def __init__(self, m=1, n=1):
        super().__init__()
        self.m = m
        self.n = n
        scalars = self.compute()
        self.scene.mlab.plot3d([0.5, 0.5], [0.5, 0.5], [-0.2, 0.1], color=(0.7, 0.7, 0.7), tube_radius=0.01, tube_sides=36)

        self.surf = self.scene.mlab.mesh(xx, yy, np.zeros_like(self.wave.real), scalars=scalars, colormap='bwr')
        # self.surf.module_manager.scalar_lut_manager.reverse_lut = True
        poly_data_normals = self.surf.module_manager.parent
        poly_data_normals.filter.compute_cell_normals = True
        poly_data_normals.filter.compute_point_normals = False
        poly_data_normals.filter.consistency = False
        poly_data_normals.filter.feature_angle = 80.0

    def compute(self):
        self.wave = 1j * np.array(np.cos(π * self.n * xx) * np.cos(π * self.m * yy), dtype=complex)
        return -self.wave.imag

    @on_trait_change("m, n")
    def update(self):
        scalars = self.compute()
        self.surf.mlab_source.z = np.zeros_like(self.wave.real)
        self.surf.mlab_source.scalars = scalars

    @mlab.animate(delay=10, ui=False, support_movie=False)
    def animate(self):
        while True:
            self.wave *= de
            self.surf.mlab_source.z = 0.1 * mode.wave.real
            yield


mode = Mode()

mode.configure_traits()
